@@ -27,6 +27,7 @@
 #include "pythonparsejob.h"
 #include "declarationbuilder.h"
 #include "helpers.h"
+#include "duchaindebug.h"
 
 #include <ktexteditor/document.h>
 
@@ -242,8 +243,6 @@ void ContextBuilder::visitGeneratorExpression(GeneratorExpressionAst* node)
 RangeInRevision ContextBuilder::comprehensionRange(Ast* node)
 {
     RangeInRevision range = editorFindRange(node, node);
-    range.start.column -= 1;
-    range.end.column += 1;
     return range;
 }
 
@@ -252,8 +251,9 @@ void ContextBuilder::visitComprehensionCommon(Ast* node)
     RangeInRevision range = comprehensionRange(node);
     Q_ASSERT(range.isValid());
     if ( range.isValid() ) {
-        DUChainWriteLocker lock(DUChain::lock());
+        DUChainWriteLocker lock;
         openContext(node, range, KDevelop::DUContext::Other);
+        qCDebug(KDEV_PYTHON_DUCHAIN) << "creating comprehension context" << node << range;
         Q_ASSERT(currentContext());
 //         currentContext()->setLocalScopeIdentifier(QualifiedIdentifier("<generator>"));
         lock.unlock();
@@ -348,27 +348,29 @@ QPair<QUrl, QStringList> ContextBuilder::findModulePath(const QString& name, con
     QDir tmp;
     QStringList leftNameComponents;
     foreach ( QUrl currentPath, searchPaths ) {
-        tmp.setPath(currentPath.path());
+        tmp.setPath(currentPath.toLocalFile());
         leftNameComponents = nameComponents;
         foreach ( QString component, nameComponents ) {
             if ( component == "*" ) {
                 // For "from ... import *", if "..." is a directory, use the "__init__.py" file
-                component = "__init__";
+                component = QStringLiteral("__init__");
             }
             else {
                 // only empty the list if not importing *, this is convenient later on
                 leftNameComponents.removeFirst();
             }
             QString testFilename = tmp.path() + "/" + component;
-            tmp.cd(component);
+
+            bool can_continue = tmp.cd(component);
             QFileInfo sourcedir(testFilename);
+            const bool dir_exists = sourcedir.exists() && sourcedir.isDir();
 
             // we can only parse those, so we don't care about anything else for now.
             // Any C modules (.so, .dll) will be ignored, and highlighted as "not found". TODO fix this
             static QStringList valid_extensions{".py", ".pyx"};
-            for(auto extension: valid_extensions) {
+            foreach ( const auto& extension, valid_extensions ) {
                 QFile sourcefile(testFilename + extension);
-                if ( ! sourcedir.exists() || ! sourcedir.isDir() || leftNameComponents.isEmpty() ) {
+                if ( ! dir_exists || leftNameComponents.isEmpty() ) {
                     // If the search cannot continue further down into a hierarchy of directories,
                     // the file matching the next name component will be returned,
                     // toegether with a list of names which must be resolved inside that file.
@@ -377,13 +379,18 @@ QPair<QUrl, QStringList> ContextBuilder::findModulePath(const QString& name, con
                         // TODO QUrl: cleanPath?
                         return qMakePair(sourceUrl, leftNameComponents);
                     }
-                    else if ( sourcedir.exists() && sourcedir.isDir() ) {
+                    else if ( dir_exists ) {
                         auto path = QUrl::fromLocalFile(testFilename + "/__init__.py");
                         // TODO QUrl: cleanPath?
                         return qMakePair(path, leftNameComponents);
                     }
                 }
             }
+            if ( ! can_continue ) {
+                // if not returned yet and the cd into the next component failed,
+                // abort and try the next search path.
+                break;
+            }
         }
     }
     return {};
